@@ -1,4 +1,5 @@
-from mathics.core.parser import parse, SingleLineFeeder
+import time
+from mathics.core.parser import parse, MathicsSingleLineFeeder
 from mathics.core.definitions import Definitions
 from mathics.core.evaluation import Evaluation
 from mathics.session import MathicsSession
@@ -11,7 +12,9 @@ def check_evaluation(str_expr: str, str_expected: str, message=""):
     result = session.evaluate(str_expr)
     expected = session.evaluate(str_expected)
 
+    print(time.asctime())
+    print(message)
     if message:
-        assert result == expected, "%s: got: %s" % (message, result)
+        assert result == expected, message
     else:
         assert result == expected
